@@ -44,11 +44,22 @@ if not ET.__name__ == "xml.etree.cElementTree":
 from upoints import (point, utils)
 
 #: Supported GPX namespace version to URI mapping
-GPX_VERSIONS = {
-  "1.0": "http://www.topografix.com/GPX/1/0",
-  "1.1": "http://www.topografix.com/GPX/1/1",
-}
 
+"""modrana:
+   the header has been expanded,
+   othervise the resulting GPX files dont validate
+"""
+GPX_VERSIONS = {
+  "1.0": {'xsi:schemaLocation':"http://www.topografix.com/GPX/1/0"}, #TODO: make also a GPX1.0 header ?
+#  "1.1": "http://www.topografix.com/GPX/1/1",
+  "1.1": {
+          'version':"1.1", 
+          'creator':"modRana - http://nlp.fi.muni.cz/projekty/gps_navigace/", 
+          'xmlns:xsi':"http://www.w3.org/2001/XMLSchema-instance", 
+          'xmlns':"http://www.topografix.com/GPX/1/1", 
+          'xsi:schemaLocation':"http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd"
+          }
+              }
 #: Default GPX version to output
 # Changing this will cause tests to fail.
 DEF_GPX_VERSION = "1.1"
@@ -72,6 +83,13 @@ def create_elem(tag, attr=None, text=None, gpx_version=DEF_GPX_VERSION,
     :return: :class:`ET.Element` wrapper with predefined namespace
 
     """
+
+    """modrana:
+       add the header to the gpx tag
+    """
+    if tag=='gpx':
+      attr = GPX_VERSIONS[gpx_version]
+
     if not attr:
         attr = {}
     try:
@@ -82,7 +100,11 @@ def create_elem(tag, attr=None, text=None, gpx_version=DEF_GPX_VERSION,
         ElementTree._namespace_map[gpx_ns] = "gpx"
         element = ElementTree.Element("{%s}%s" % (gpx_ns, tag), attr)
     else:
-        element = ET.Element("{%s}%s" % (gpx_ns, tag), attr)
+#        element = ET.Element("{%s}%s" % (gpx_ns, tag), attr)
+        # modrana:
+        # this fixes the resulting GPX not being valid becuase of unknown
+        # namespace prefixes
+        element = ET.Element("%s" % (tag), attr)
     if text:
         element.text = text
     return element
@@ -195,7 +217,13 @@ class _GpxElem(point.TimedPoint):
         if self.elevation:
             element.append(elementise("ele", None, str(self.elevation)))
         if self.time:
-            element.append(elementise("time", None, self.time.isoformat()))
+#            element.append(elementise("time", None, self.time.isoformat()))
+            """modrana:
+               the previous format was using current time type sufix
+               this was chcnged just to "Z", as demanded by the spcification
+               the time should be just UTC, not local time
+            """
+            element.append(elementise("time", None, time.strftime("%Y-%m-%dT%H:%M:%SZ")))
         return element
 
 
@@ -493,11 +521,11 @@ class _GpxMeta(object):
                 metadata.append(element)
         element = elementise("time", None)
         if isinstance(self.time, (time.struct_time, tuple)):
-            element.text = time.strftime("%Y-%m-%dT%H:%M:%S%z", self.time)
+            element.text = time.strftime("%Y-%m-%dT%H:%M:%SZ", self.time) # GPX documentation states, the Z on the end should be capital
         elif isinstance(self.time, utils.Timestamp):
             element.text = self.time.isoformat()
         else:
-            element.text = time.strftime("%Y-%m-%dT%H:%M:%S%z")
+            element.text = time.strftime("%Y-%m-%dT%H:%M:%SZ") # GPX documentation states, the Z on the end should be capital
         metadata.append(element)
         if self.keywords:
             metadata.append(elementise("keywords", None, self.keywords))
@@ -820,17 +848,33 @@ class Trackpoints(_SegWrap):
             try:
                 accepted_gpx = {gpx_version: GPX_VERSIONS[gpx_version]}
             except KeyError:
+#                print GPX_VERSIONS
                 raise KeyError("Unknown GPX version `%s'" % gpx_version)
         else:
             accepted_gpx = GPX_VERSIONS
 
         for version, namespace in accepted_gpx.items():
             logging.info("Searching for GPX v%s entries" % version)
+            """
+            modrana:
+            get the namespace from the tag
+            this change was needed, because the old way of prefix handling was 
+            changed to get rid of prefixes in output
+            TODO: do this more cleanly"""
 
+            # get just the namespace string
+            namespace = data.getroot().tag.split('}')[0]
+            namespace = namespace[1:]
+
+            # create a qualified name containing the namespace
             gpx_elem = lambda name: ET.QName(namespace, name).text
             metadata = data.find("//" + gpx_elem("metadata"))
+#            print namespace
+#            print data.getroot().tag
+#            print gpx_elem("metadata")
+#            print metadata
             if metadata:
-                self.metadata.import_metadata(metadata)
+                self.metadata.import_metadata(metadata)               
             segment_elem = "//" + gpx_elem("trkseg")
             trackpoint_elem = gpx_elem("trkpt")
             name_elem = gpx_elem("name")
@@ -838,6 +882,10 @@ class Trackpoints(_SegWrap):
             elev_elem = gpx_elem("ele")
             time_elem = gpx_elem("time")
 
+
+#            print "segment?"
+#            print data.findall(segment_elem)
+
             for segment in data.findall(segment_elem):
                 points = point.TimedPoints()
                 for trackpoint in segment.findall(trackpoint_elem):
